# include <command_rmsd.h>
# include <global.h>
# include <pdb.h>

CommandRmsd::CommandRmsd(vector<string> flags)
	:Command("RMSD")
{
	caflag = 0;
	backbone4flag=0;
	backbone3flag=0;
	sidechainflag=0;
	for(int i=0;i<flags.size();i++)
	{
		if(flags[i]=="ca") caflag=1;
		else
		if(flags[i]=="backbone") backbone4flag=1;
		else
		if(flags[i]=="backbone4") backbone4flag=1;
		else
		if(flags[i]=="backbone3") backbone3flag=1;
		else
		if(flags[i]=="sidechain") sidechainflag=1;
		else
		{
			Log("Unknown keyword "+flags[i]+" for Rmsd option");
			psf_error("Unknown keyword "+flags[i]+" for Rmsd option");
		}
	}	
}


void	CommandRmsd::Run()
{
	Team *rmsd_team = team;
	if(rmsd_team == NULL)  Error(SeqError("rmsd"));
	if(pdbpos.size()==0)
		psf_error("CAN NOT RUN RMSD WITHOUT pdb FILE (OPTION -pdb) ");
	IntVector AtomPos;
	string chain;
	vector<string> chainlist;
	IntVector ChainPos;
	
	extern vector<atom> table;
	for(int i=0;i<table.size();i++)
	{
		if(isHydro(table[i]))  continue;
		if(rmsd_team->find(table[i],chain))
		{
			int flag=0;
			if(!caflag && !backbone3flag && !backbone4flag && !sidechainflag) flag=1;
			else
			{
			if(table[i].atom_name=="CA" && caflag) flag=1;

			if(isBackbone4(table[i]) && backbone4flag) flag=1;
			if(isBackbone3(table[i]) && backbone3flag) flag=1;
			if(isSidechain(table[i]) && sidechainflag) flag=1;

			if(flag)
			{
				AtomPos.push_back(i);
				int ifound=-1;
				for(int j=0;j<chainlist.size();j++)
				{
					if(chainlist[j]==table[i].chain_id) {ifound=j;break;}
				}
				if(ifound==-1)
				{
					ifound=chainlist.size();
					chainlist.push_back(table[i].chain_id);
				}
				Log(printAtom3(table[i]));
				ChainPos.push_back(ifound);
			}
		}
	}
	}
	if(chainlist.size()==0) return;
	IntVector F;
	vector<DoubleVector> D;
	IntVector AtomPos2;
	int chain_count = chainlist.size();
	D.resize(chain_count);
	for(int i=0;i<chain_count;i++)
	{
		AtomPos2.resize(0);
		for(int j=0;j<AtomPos.size();j++)
			if(table[AtomPos[j]].chain_id==chainlist[i]) AtomPos2.push_back(AtomPos[j]);
		dcd->getRmsd(AtomPos2,pdbpos,F,D[i]);
	}
	DoubleVector totalD;
	if(chainlist.size()>1)	
	dcd->getRmsd(AtomPos,pdbpos,F,totalD);
	FILE *fp=fopen("rmsd.dat","w");
	if(!fp) Error(WriteError("rmsd.dat"));
	Print(fp,"#");
	Print(fp,printString("frame",7)+" ");
	for(int i=0;i<chainlist.size();i++)
		Print(fp,printString(chainlist[i],8)+" ");
	if(chainlist.size()>1)
	PrintLine(fp,printDistanceHeader("Total"));
	else
	PrintLine(fp,"");
	for(int i=0;i<F.size();i++)
	{
		Print(fp,printFrame(F[i])+" ");
		for(int j=0;j<chainlist.size();j++)
		{
			Print(fp,printDistance(D[j][i])+" ");
		}
		if(chainlist.size()>1)
		PrintLine(fp,printDistance(totalD[i]));
		else
		PrintLine(fp,"");
	}
	fclose(fp);
	if(smooth_flag)
	{
		IntVector sf;
		vector<DoubleVector> sd;
		DoubleVector smooth_totalD;
		sd.resize(chainlist.size());
		for(int i=0;i<chainlist.size();i++) makeSmooth(F,D[i],sf,sd[i],smooth_start,smooth_step);
		makeSmooth(F,totalD,sf,smooth_totalD,smooth_start,smooth_step);
		fp=fopen("rmsd.sda","w");
		Print(fp,"#");
		Print(fp,printString("frame",7)+" ");
		for(int i=0;i<chainlist.size();i++)
			Print(fp,printString(chainlist[i],8)+" ");
		PrintLine(fp,printDistanceHeader("Total"));
		for(int i=0;i<sf.size();i++)
		{
			Print(fp,printFrame(sf[i])+" ");
			for(int j=0;j<chainlist.size();j++)
			{
				Print(fp,printDistance(sd[j][i])+" ");
			}
			if(chainlist.size()>1)
			PrintLine(fp,printDistance(smooth_totalD[i]));
			else PrintLine(fp,"");
		}
		fclose(fp);
	}
	fp=fopen("rmsd.stat","w");
	fclose(fp);
	for(int i=0;i<chainlist.size();i++)
	{
		PrintStatFile("rmsd.stat","a",F.size()/statcount,"#Rmsd statistics for chain "+chainlist[i],F,D[i]);
	}
	if(chainlist.size()>1)
	PrintStatFile("rmsd.stat","a",F.size()/statcount,"#Total rmsd statistics",F,totalD);
	fp=fopen("rmsd.hist","w");
	typedef vector<HistStruct> histd;
	vector<histd> st;
	st.resize(chainlist.size());
	Data dmin,dmax,davg,dstd;
	for(int i=0;i<chainlist.size();i++)
	{
		getVectorStatistics(D[i],dmin,dmax,davg,dstd);
		makeHist(D[i],st[i],dmin,dmax,bindist);
	}
	
	for(int j=0;j<st[0].size();j++)
	{
		for(int i=0;i<chainlist.size();i++)
		{
		if(i==0) Print(fp,printDistance(st[i][j].value)+" ");
		Print(fp,printPercent(st[i][j].percent)+" "+printFrame(st[i][j].count)+" ");
		}
		PrintLine(fp,"");
	}
	fclose(fp);
}
